@@ -2,7 +2,7 @@
 The ibidas module contains all main functions for working with ibidas objects.
 """
 
-__all__ = ["Rep","Read","Import","Connect","_","CyNetwork",'Unpack',
+__all__ = ["Rep","Read","Import","Connect","_","CyNetwork",'Unpack', "Addformat",
            "Array","Tuple","Combine","HArray",
            "Stack","Intersect","Union","Except","Difference",
            "Pos","Argsort","Rank","IsMissing","CumSum",
@@ -13,11 +13,12 @@ __all__ = ["Rep","Read","Import","Connect","_","CyNetwork",'Unpack',
            "Sum",
            "Count","Match",
            "Broadcast","CreateType","MatchType",
-           "newdim","Missing",
+           "newdim","NEWDIM","LCDIM","LASTCOMMONDIM","COMMON_NAME",'COMMON_POS', "Missing",
            "Corr","In","Contains",
            "Fetch","Serve","Get","Alg",
-           "Load","Save",'newdim',
+           "Load","Save",
            "Invert","Abs", "Negative",
+           "Add","Subtract","Multiply","Modulo","Divide","FloorDivide","And","Or","Xor","Power","Equal","NotEqual","LessEqual","Less","GreaterEqual","Greater","Each"
            ]
 
 from utils import delay_import
@@ -27,7 +28,8 @@ from utils.missing import Missing
 from utils.infix import Infix
 from itypes import createType as CreateType, matchType as MatchType
 from wrappers.python import Rep
-from representor import newdim
+from constants import *
+from repops import Detect
 import repops_dim 
 from repops_multi import Broadcast, Combine, Sort
 import repops_slice 
@@ -70,27 +72,82 @@ Abs = repops.delayable()(repops_funcs.Abs)
 Negative = repops.delayable()(repops_funcs.Negative)
 
 
+Add = repops.delayable()(repops_funcs.Add)
+Subtract = repops.delayable()(repops_funcs.Subtract)
+Multiply = repops.delayable()(repops_funcs.Multiply)
+Modulo= repops.delayable()(repops_funcs.Modulo)
+Divide = repops.delayable()(repops_funcs.Divide)
+FloorDivide = repops.delayable()(repops_funcs.FloorDivide)
+And = repops.delayable()(repops_funcs.And)
+Or = repops.delayable()(repops_funcs.Or)
+Xor = repops.delayable()(repops_funcs.Xor)
+Power = repops.delayable()(repops_funcs.Power)
+Equal = repops.delayable()(repops_funcs.Equal)
+NotEqual = repops.delayable()(repops_funcs.NotEqual)
+LessEqual = repops.delayable()(repops_funcs.LessEqual)
+Less = repops.delayable()(repops_funcs.Less)
+GreaterEqual = repops.delayable()(repops_funcs.GreaterEqual)
+Greater = repops.delayable()(repops_funcs.Greater)
+Each = repops.delayable()(repops_funcs.Each)
+
+
 HArray = repops.delayable(nsources=UNDEFINED)(repops_slice.HArray)
 Tuple = repops.delayable(nsources=UNDEFINED)(repops_slice.Tuple)
 Array = repops.delayable()(repops_dim.Array)
 
-def Import(url, **kwargs):
-    format = kwargs.pop('format','tsv')
 
-    if(format == 'tsv'):
-        from wrappers.tsv import TSVRepresentor
-        return TSVRepresentor(url, **kwargs) 
-    if(format == 'matrix_tsv'):
-        from wrappers.matrix_tsv import MatrixTSVRepresentor
-        return MatrixTSVRepresentor(url, **kwargs) 
-    elif(format == 'xml'):
-        from wrappers.xml_wrapper import XMLRepresentor
-        return XMLRepresentor(url, **kwargs) 
-    elif(format == 'psimi'):
-        from wrappers.psimi import read_psimi
-        return read_psimi(url, **kwargs)
-    else:
+
+def fimport_tsv(url, **kwargs):
+    from wrappers.tsv import TSVRepresentor
+    return TSVRepresentor(url, **kwargs) 
+
+def fimport_matrixtsv(url, **kwargs):
+    from wrappers.matrix_tsv import MatrixTSVRepresentor
+    return MatrixTSVRepresentor(url, **kwargs)
+
+def fimport_xml(url, **kwargs):
+    from wrappers.xml_wrapper import XMLRepresentor
+    return XMLRepresentor(url, **kwargs) 
+
+def fimport_psimi(url, **kwargs):
+    from wrappers.psimi import read_psimi
+    return read_psimi(url, **kwargs)
+
+def fimport_fasta(url, **kwargs):
+    from wrappers.fasta import read_fasta;
+    return read_fasta(url, **kwargs);
+
+
+formats = { 'tsv' : fimport_tsv, 'csv' : fimport_tsv,
+            'tsv_matrix' : fimport_matrixtsv,
+	    'xml' : fimport_xml,
+	    'psimi' : fimport_psimi,
+	    'fasta' : fimport_fasta, 'fa' : fimport_fasta, 'fas' : fimport_fasta
+	  };
+
+def Addformat(ext, read_fn):
+    formats[ext] = read_fn;
+
+def Import(url, **kwargs):
+
+  from os.path import splitext;
+  detect=kwargs.pop('detect', True);
+
+  base = url;
+
+  while True:
+    (base, ext) = splitext(base);
+    ext = ext.split('.')[1] if ext else 'tsv';
+    format = kwargs.pop('format', ext).lower();
+
+    if not format:
         raise RuntimeError("Unknown format specified")
+    if format not in formats:
+        continue;
+    else:
+        data = formats[format](url, **kwargs);
+        return data.Detect() if detect else data;
+
 Read = Import
 
 def Connect(url, **kwargs):
