@@ -13,52 +13,64 @@ _delay_import_(globals(),"itypes","rtypes","dimpaths","casts")
 class ProjectDim(repops.UnaryOpRep):
     def _sprocess(self, source, name):
         nslices = [slice for slice in source._slices if slice.dims.hasName(name)]
-        assert nslices,  "Cannot find matching slices for dimension: " + str(name)
+        if not nslices:
+            raise AttributeError, "Cannot find matching slices for dimension '" + str(name) + "'"
         self._initialize(tuple(nslices))
 
 class ProjectBookmark(repops.UnaryOpRep):
     def _sprocess(self, source, name):
         nslices = [slice for slice in source._slices if name in slice.bookmarks]
-        assert nslices,  "Cannot find matching slices for bookmark: " + str(name)
+        if not nslices:
+            raise AttributeError, "Cannot find matching slices for bookmark '" + str(name) + "'"
         self._initialize(tuple(nslices))
 
+class RequestUnaryOpRep(repops.UnaryOpRep):
+    def __init__(self,source,*args, **kwds):
+        self._req_sources = []
+        repops.UnaryOpRep.__init__(self,source,*args, **kwds)
 
-class To(repops.UnaryOpRep):
+
+class To(RequestUnaryOpRep):
     def _sprocess(self, source, *slicesel, **kwargs):
         do = kwargs.pop("Do")
         assert not kwargs, "Unknown parameters: " + str(kwargs)
 
+        if not self._req_sources:
+            for ssel in slicesel:
+                r = source.Get(ssel) 
+                #assert len(r._slices) == 1, "To action can only be applied to single slices"
+                all_pos = []
+                for slice in r._slices:
+                    #slice = r._slices[0]
+                    assert slice in source._slices, "Selected slice in to should not be operated upon"
+                    pos = source._slices.index(slice)
+                    all_pos.append(pos)
+
+                if(isinstance(do, context.Context)):
+                    r = context._apply(do, r)
+                else:
+                    r = do(r)
+
+                self._req_sources.append((all_pos, r))
+
         nslices = list(source._slices)
-        for ssel in slicesel:
-            r = source.Get(ssel) 
-            #assert len(r._slices) == 1, "To action can only be applied to single slices"
-            all_pos = []
-            for slice in r._slices:
-                #slice = r._slices[0]
-                assert slice in source._slices, "Selected slice in to should not be operated upon"
-                pos = source._slices.index(slice)
-                all_pos.append(pos)
-
-            if(isinstance(do, context.Context)):
-                r = context._apply(do, r)
-            else:
-                r = do(r)
-
-            
+        for all_pos, r in self._req_sources:
+            if not r._slicesKnown():
+                return 
             for slice,pos in zip(r._slices, all_pos):
                 nslices[pos] = slice
 
         self._initialize(tuple(nslices))
        
 
-class Project(repops.UnaryOpRep):
+class Project(RequestUnaryOpRep):
     def __init__(self,source,*args, **kwds):
-        self._project_sources = []
+        self._req_sources = []
         repops.UnaryOpRep.__init__(self,source,*args, **kwds)
 
-    def _getUsedSourceSlicesSet(self,project_sources):
+    def _getUsedSourceSlicesSet(self,req_sources):
         nslices = []
-        for name, elem in project_sources:
+        for name, elem in req_sources:
             if isinstance(elem, representor.Representor):
                 nslices.extend(elem._slices)
             else:
@@ -69,14 +81,14 @@ class Project(repops.UnaryOpRep):
         cur_slices = self._source._slices
         
         nslices = []
-        if(not self._project_sources):
-            project_sources = []
+        if(not self._req_sources):
+            req_sources = []
             for name, elem in chain(zip([None] * len(args),args),kwds.iteritems()):
                 if(isinstance(elem, context.Context)):
                     elem = context._apply(elem, self._source)
                 elif(isinstance(elem, basestring)):
                     if(elem == "~"):
-                        used_slices = set([slice.name for slice in self._getUsedSourceSlicesSet(project_sources)])
+                        used_slices = set([slice.name for slice in self._getUsedSourceSlicesSet(req_sources)])
                         nelem = [slice for slice in cur_slices if slice.name not in used_slices]
                     elif(elem == "#"):
                         common_dims = set([slice.dims for slice in cur_slices])
@@ -93,9 +105,9 @@ class Project(repops.UnaryOpRep):
                                nelem = UnpackTuple._apply(cur_slices[0],elem)
                             else:
                                nelem = [slice for slice in cur_slices if elem in slice.bookmarks]
-                               assert len(nelem) > 0, "Could not find (unique) matching slice/bookmark for name: " + elem
-                        else:
-                            assert len(nelem) > 0, "Could not find (unique) matching slice for name: " + elem
+                        
+                        if not nelem:
+                            raise AttributeError, "Cannot find attribute '" + elem + "'"
                     elem = nelem
                 elif(isinstance(elem, representor.Representor)):
                     pass
@@ -109,22 +121,25 @@ class Project(repops.UnaryOpRep):
                 else:
                     elem = util.select(cur_slices, elem)
                    
-                project_sources.append((name,elem))
-            self._project_sources = project_sources
+                req_sources.append((name,elem))
+            self._req_sources = req_sources
      
         
-        for name,elem in self._project_sources:
+        for name,elem in self._req_sources:
             if(isinstance(elem, representor.Representor)):
                 if not elem._slicesKnown():
                     return 
                 elem = [slice for slice in elem._slices]
 
             if(name):
-                assert len(elem) == 1, "Could not find a (unique) matching slice for " + name
+                if not len(elem) == 1:
+                    raise AttributeError, "Cannot find unique slice for '" + name + "', found: " + str(elem)
                 nslices.append(ops.ChangeNameOp(elem[0],name))
             else:
                 nslices.extend(elem)
-        assert nslices, "No slices found with: " + str(args) + " and " + str(kwds)
+        
+        if not nslices:                
+            raise AttributeError,  "No slices found with: " + str(args) + " and " + str(kwds)
         
         return self._initialize(tuple(nslices)) 
     
@@ -159,8 +174,12 @@ class UnpackTuple(repops.UnaryOpRep):
         if not source._typesKnown():
             return
 
-        assert len(source._slices) == 1, \
-                "Unpacking tuples can only be done on single slices"
+        if not len(source._slices) == 1:                
+            if(name):
+                raise AttributeError, "Asked to unpack tuple attribute '" + \
+                    name + "', but cannot find a tuple."
+            else:
+                raise AttributeError, "No tuple to unpack"
         slice = source._slices[0]
         nslices = self._apply(slice,name,unpack=unpack)
         return self._initialize(tuple(nslices))
@@ -169,10 +188,10 @@ class UnpackTuple(repops.UnaryOpRep):
     def _apply(cls, slice, name=None, unpack=True):
         if(not isinstance(slice.type, rtypes.TypeTuple)):
             if(name):
-                raise RuntimeError, "Asked to unpack tuple attribute " + \
-                    name + " but cannot find a tuple."
+                raise AttributeError, "Asked to unpack tuple attribute '" + \
+                    name + "', but cannot find a tuple"
             else:
-                raise RuntimeError, "No tuple to unpack"
+                raise AttributeError, "No tuple to unpack"
 
         if(name is None):
             nslices = [ops.UnpackTupleOp(slice, idx) for idx in range(len(slice.type.subtypes))]
@@ -185,7 +204,7 @@ class UnpackTuple(repops.UnaryOpRep):
                 try:
                     idx = slice.type.fieldnames.index(name)
                 except ValueError:
-                    raise RuntimeError, "Name: " + str(name) + " not in tuple"
+                    raise AttributeError, "Cannot unpack, as tuple has no field '" + str(name) + "'"
             nslices = [ops.UnpackTupleOp(slice, idx)]
         
         if(unpack):
