@@ -17,7 +17,7 @@ import gc, sys
 import itertools
 import csv
 import cutils
-
+from logging import error,warning,info,debug
 
 #objs: objs to find names for
 #req_objs: objs which should be in same frame
@@ -66,7 +66,7 @@ def save_rep(r, filename):
 def save_csv(r, filename):
     f = open(filename,'wb')
     r= r.Array(tolevel=1)
-    data = r.Cast(str).ToPython();
+    data = r.Cast(str).Tuple().ToPython();
     if filename.endswith('tsv'):
         w = csv.writer(f,delimiter='\t');
     else:
@@ -276,7 +276,7 @@ class farray(numpy.ndarray):
     def __eq__(self, other):
         return (self.__class__ is other.__class__ and
                    self.shape == other.shape and
-             numpy.equal(self.view(numpy.ndarray),other.view(numpy.ndarray)).all())
+             (self.view(numpy.ndarray) == other.view(numpy.ndarray)).all())
 
     def __ne__(self, other):
         return not self.__eq__(other)
